
**_This project is currently in early-access deep-alpha testing phase. Do not
rely on it for anything._**

-----

# cryptic-net

The cryptic-net project provides the foundation for an **autonomous community
cloud infrastructure**.

This project targets communities of individuals, where certain members of the
community would like to host services and applications from servers running in
their homes or offices. These servers can range from simple Raspberry Pis to
full-sized home PCs.

The core components of cryptic-net, currently, are:

* A VPN which enables direct peer-to-peer communication. Even if most hosts in
  the network are on a private LAN (e.g. their home WiFi network) or have a
  dynamic IP, they can still communicate directly with each other.

* An S3-compatible network filesystem. Each users can provide as much storage as
  they care to, if any. Stored data is sharded and replicated across all hosts
  that choose to provide storage.

* A DNS server which provides automatic host and service (coming soon) discovery
  within the network.

These components are wrapped into a single binary, with all setup being
automated. cryptic-net takes "just works" very seriously.

Participants are able to build upon these foundations to host services for
themselves and others. They can be assured that their communications are private
and their storage is reliable, all with zero administrative overhead and zero
third parties involved.

## Documentation

cryptic-net users fall into different roles, depending on their level of
involvement and expertise within their particular network. The documentation for
cryptic-net is broken down by these categories, so that the reader can easily
decide which documents they need to care about.

### User Docs

Users are participants who use network resources, but do not provide any network
or storage resources themselves. Users may be accessing the network from a
laptop, and so are not expected to be online at any particular moment.

Documentation for users:

* [Getting Started](docs/user/getting-started.md)
* [Creating a daemon.yml File](docs/user/creating-a-daemonyml-file.md)
* [Using DNS](docs/user/using-dns.md) (advanced)
* Restic example (TODO)

### Operator Docs

Operators are users who own a dedicated host which they can expect to be
always-online (to the extent that's possible in a residential environment).
Operator hosts will need at least one of the following to be useful:

* A static public IP, or a dynamic public IP with [dDNS][ddns] set up.

* At least 100GB of unused storage which can be reserved for the network.

Operators are expected to be familiar with server administration, and to not be
afraid of a terminal.

Documentation for operators:

* [Contributing Storage](docs/operator/contributing-storage.md)
* [Contributing a Lighthouse](docs/operator/contributing-a-lighthouse.md)
* [Managing garage](docs/operator/managing-garage.md)
* [Firewalls](doc/operator/firewall.md)

[ddns]: https://www.cloudflare.com/learning/dns/glossary/dynamic-dns/

### Admin Docs

Admins are users who control membership within the network. They are likely
operators as well.

Documentation for admins:

* [Creating a New Network](docs/admin/creating-a-new-network.md)
* [Adding a Host to the Network](docs/admin/adding-a-host-to-the-network.md)
* Removing a Host From the Network (TODO)

### Dev Docs

Devs may or may not be users in any particular cryptic network. They instead are
those who work on the actual code for cryptic-net.

Documentation for devs:

* [Design Principles](docs/dev/design-principles.md)
* [`cryptic-net daemon` process tree](docs/dev/daemon-process-tree.svg): Diagram
  describing the [pmux](https://code.betamike.com/cryptic-io/pmux) process tree
  created by `cryptic-net daemon` at runtime.
* [Rebuilding Documentation](docs/dev/rebuilding-documentation.md)

## Misc

Besides documentation, there are a few other pages which might be useful:

* [Roadmap][roadmap]
* [Glossary](docs/glossary.md)

[roadmap]: docs/roadmap.md
