# Contributing Storage

If your host machine can be reasonably sure of being online most, if not all, of
the time, and has 100GB or more of unused drive space you'd like to contribute
to the network, then this document is for you.

## Create `daemon.yml`

First, if you haven't already, [create a `daemon.yml`
file](../user/creating-a-daemonyml-file.md). This will be used to
configure your `cryptic-net daemon` process with the storage locations and
capacities you want to contribute.

## Edit `daemon.yml`

Open your `daemon.yml` file  in a text editor, and find the
`storage.allocations` section.

Each allocation in the allocations list describes the space being contributed
from a single physical drive. If you only have one drive then you will only need
one allocation listed.

The comments in the file should be self-explanatory, but ask your admin if you
need any clarification.

Here are an example set of allocations for a host which is contributing space
from two separate drives:

```
storage:
  allocations:

    # 1.2 TB are being shared from drive1
    - data_path: /mnt/drive1/cryptic-net/data
      meta_path: /mnt/drive1/cryptic-net/meta
      capacity: 1200

    # 100 GB (the minimum) are being shared from drive2
    - data_path: /mnt/drive2/cryptic-net/data
      meta_path: /mnt/drive2/cryptic-net/meta
      capacity: 100
```

## Set Up Your Firewall

See the doc on [Firewalls](./firewalls.md), to be sure that your host's firewall
is properly set up for providing storage.

## Restart the Daemon

With the `daemon.yml` configured, you should restart your `cryptic-net daemon`
process.

The `cryptic-net daemon` will automatically allow the ports used for your
storage allocations in the vpn firewall.

## Further Reading

cryptic-net uses the [garage][garage] project for its storage system. See the
[Managing Garage](managing-garage.md) document for more
information on how to interact directly with the garage instance being run by
cryptic-net.

[garage]: https://garagehq.deuxfleurs.fr/documentation/quick-start/
