package main

import (
	"context"
	"cryptic-net/bootstrap"
	"cryptic-net/daemon"
	"cryptic-net/garage"
	"fmt"
	"time"
)

func mergeDaemonConfigIntoBootstrap(
	hostBootstrap bootstrap.Bootstrap,
	daemonConfig daemon.Config,
) (
	bootstrap.Bootstrap, error,
) {

	host := bootstrap.Host{
		Name: hostBootstrap.HostName,
		Nebula: bootstrap.NebulaHost{
			SignedPublicCredentials: hostBootstrap.Nebula.SignedPublicCredentials,
			PublicAddr:              daemonConfig.VPN.PublicAddr,
		},
	}

	if allocs := daemonConfig.Storage.Allocations; len(allocs) > 0 {

		for _, alloc := range allocs {

			id, err := garage.InitAlloc(alloc.MetaPath)
			if err != nil {
				return bootstrap.Bootstrap{}, fmt.Errorf("initializing alloc at %q: %w", alloc.MetaPath, err)
			}

			host.Garage.Instances = append(host.Garage.Instances, bootstrap.GarageHostInstance{
				ID:        id,
				RPCPort:   alloc.RPCPort,
				S3APIPort: alloc.S3APIPort,
			})
		}
	}

	hostBootstrap.Hosts[host.Name] = host

	if err := writeBootstrapToDataDir(hostBootstrap); err != nil {
		return bootstrap.Bootstrap{}, fmt.Errorf("writing bootstrap file: %w", err)
	}

	return hostBootstrap, nil
}

func doOnce(ctx context.Context, fn func(context.Context) error) error {
	for {
		if err := fn(ctx); err == nil {
			return nil
		} else if ctxErr := ctx.Err(); ctxErr != nil {
			return ctxErr
		}

		time.Sleep(250 * time.Millisecond)
	}
}
