package main

import (
	"context"
	"fmt"
	"os"
	"os/signal"
	"path/filepath"
	"syscall"

	"github.com/adrg/xdg"
)

// The purpose of this binary is to act as the entrypoint of the cryptic-net
// process. It processes the command-line arguments which are passed in, and
// then passes execution along to an appropriate binary housed in AppDir/bin
// (usually a bash script, which is more versatile than a go program).

func getAppDirPath() string {
	appDirPath := os.Getenv("APPDIR")
	if appDirPath == "" {
		appDirPath = "."
	}
	return appDirPath
}

var (
	envAppDirPath     = getAppDirPath()
	envRuntimeDirPath = filepath.Join(xdg.RuntimeDir, "cryptic-net")
	envDataDirPath    = filepath.Join(xdg.DataHome, "cryptic-net")
)

func main() {

	ctx, cancel := context.WithCancel(context.Background())

	signalCh := make(chan os.Signal, 2)
	signal.Notify(signalCh, syscall.SIGINT, syscall.SIGTERM)

	go func() {
		sig := <-signalCh
		cancel()
		fmt.Fprintf(os.Stderr, "got signal %v, will exit gracefully\n", sig)

		sig = <-signalCh
		fmt.Fprintf(os.Stderr, "second interrupt signal %v received, force quitting, there may be zombie children left behind, good luck!\n", sig)

		os.Stderr.Sync()
		os.Exit(1)
	}()

	err := subCmdCtx{
		args: os.Args[1:],
		ctx:  ctx,
	}.doSubCmd(
		subCmdAdmin,
		subCmdDaemon,
		subCmdGarage,
		subCmdHosts,
		subCmdVersion,
	)

	if err != nil {
		panic(err)
	}
}
